@@ -50,6 +50,8 @@ void ZrtpQueue::init()
     zrtpUserCallback = NULL;
     enableZrtp = false;
     started = false;
+    mitmMode = false;
+    enableParanoidMode = false;
     zrtpEngine = NULL;
     senderZrtpSeqNo = 1;
 
@@ -69,8 +71,7 @@ ZrtpQueue::~ZrtpQueue() {
 }
 
 int32_t
-ZrtpQueue::initialize(const char *zidFilename, bool autoEnable,
-                     ZrtpConfigure* config)
+ZrtpQueue::initialize(const char *zidFilename, bool autoEnable, ZrtpConfigure* config)
 {
     int32_t ret = 1;
 
@@ -83,6 +84,8 @@ ZrtpQueue::initialize(const char *zidFilename, bool autoEnable,
     }
     enableZrtp = autoEnable;
 
+    config->setParanoidMode(enableParanoidMode);
+
     if (staticTimeoutProvider == NULL) {
         staticTimeoutProvider = new TimeoutProvider<std::string, ZrtpQueue*>();
         staticTimeoutProvider->start();
@@ -104,7 +107,7 @@ ZrtpQueue::initialize(const char *zidFilename, bool autoEnable,
     }
     if (ret > 0) {
         const uint8_t* ownZid = zf->getZid();
-        zrtpEngine = new ZRtp((uint8_t*)ownZid, (ZrtpCallback*)this, clientIdString, config);
+        zrtpEngine = new ZRtp((uint8_t*)ownZid, (ZrtpCallback*)this, clientIdString, config, mitmMode, signSas);
     }
     if (configOwn != NULL) {
         delete configOwn;
@@ -196,9 +199,7 @@ ZrtpQueue::takeInDataPacket(void)
 }
 
 size_t
-ZrtpQueue::rtpDataPacket(unsigned char* buffer, int32 rtn,
-                         InetHostAddress network_address,
-                         tpport_t transport_port)
+ZrtpQueue::rtpDataPacket(unsigned char* buffer, int32 rtn, InetHostAddress network_address, tpport_t transport_port)
 {
      // Special handling of padding to take care of encrypted content.
     // In case of SRTP the padding length field is also encrypted, thus
@@ -295,12 +296,8 @@ ZrtpQueue::rtpDataPacket(unsigned char* buffer, int32 rtn,
         network_address, transport_port) &&
         recordReception(*sourceLink,*packet,recvtime) ) {
         // now the packet link is linked in the queues
-        IncomingRTPPktLink* packetLink =
-                new IncomingRTPPktLink(packet,
-                                       sourceLink,
-                                       recvtime,
-                                       packet->getTimestamp() -
-                                               sourceLink->getInitialDataTimestamp(),
+        IncomingRTPPktLink* packetLink = new IncomingRTPPktLink(packet, sourceLink, recvtime,
+                                       packet->getTimestamp() - sourceLink->getInitialDataTimestamp(),
                                        NULL,NULL,NULL,NULL);
         insertRecvPacket(packetLink);
     } else {
@@ -317,7 +314,6 @@ ZrtpQueue::rtpDataPacket(unsigned char* buffer, int32 rtn,
     return rtn;
 }
 
-
 bool
 ZrtpQueue::onSRTPPacketError(IncomingRTPPkt& pkt, int32 errorCode)
 {
@@ -378,9 +374,11 @@ int32_t ZrtpQueue::sendDataZRTP(const unsigned char *data, int32_t length) {
 
 bool ZrtpQueue::srtpSecretsReady(SrtpSecret_t* secrets, EnableSecurity part)
 {
-    CryptoContext* pcc;
     CryptoContext* recvCryptoContext;
     CryptoContext* senderCryptoContext;
+    CryptoContextCtrl* recvCryptoContextCtrl;
+    CryptoContextCtrl* senderCryptoContextCtrl;
+
     int cipher;
     int authn;
     int authKeyLen;
@@ -421,6 +419,17 @@ bool ZrtpQueue::srtpSecretsReady(SrtpSecret_t* secrets, EnableSecurity part)
                     authKeyLen,                              // authentication key len
                     secrets->initSaltLen / 8,                // session salt len
                     secrets->srtpAuthTagLen / 8);            // authentication tag lenA
+            senderCryptoContextCtrl = new CryptoContextCtrl(0,
+                  cipher,                                    // encryption algo
+                  authn,                                     // authtication algo
+                  (unsigned char*)secrets->keyInitiator,     // Master Key
+                  secrets->initKeyLen / 8,                   // Master Key length
+                  (unsigned char*)secrets->saltInitiator,    // Master Salt
+                  secrets->initSaltLen / 8,                  // Master Salt length
+                  secrets->initKeyLen / 8,                   // encryption keyl
+                  authKeyLen,                                // authentication key len
+                  secrets->initSaltLen / 8,                  // session salt len
+                  secrets->srtpAuthTagLen / 8);              // authentication tag len
         }
         else {
             senderCryptoContext = new CryptoContext(
@@ -437,21 +446,28 @@ bool ZrtpQueue::srtpSecretsReady(SrtpSecret_t* secrets, EnableSecurity part)
                     authKeyLen,                              // authentication key len
                     secrets->respSaltLen / 8,                // session salt len
                     secrets->srtpAuthTagLen / 8);            // authentication tag len
+            senderCryptoContextCtrl = new CryptoContextCtrl(0,
+                  cipher,                                    // encryption algo
+                  authn,                                     // authtication algo
+                  (unsigned char*)secrets->keyResponder,     // Master Key
+                  secrets->respKeyLen / 8,                   // Master Key length
+                  (unsigned char*)secrets->saltResponder,    // Master Salt
+                  secrets->respSaltLen / 8,                  // Master Salt length
+                  secrets->respKeyLen / 8,                   // encryption keyl
+                  authKeyLen,                                // authentication key len
+                  secrets->respSaltLen / 8,                  // session salt len
+                  secrets->srtpAuthTagLen / 8);              // authentication tag len
         }
         if (senderCryptoContext == NULL) {
             return false;
         }
-        // Create a SRTP crypto context for real SSRC sender stream.
-        // Note: key derivation can be done at this time only if the
-        // key derivation rate is 0 (disabled). For ZRTP this is the
-        // case: the key derivation is defined as 2^48
-        // which is effectively 0.
-        pcc = senderCryptoContext->newCryptoContextForSSRC(getLocalSSRC(), 0, 0L);
-        if (pcc == NULL) {
-            return false;
-        }
-        pcc->deriveSrtpKeys(0L);
-        setOutQueueCryptoContext(pcc);
+        // Insert the Crypto templates (SSRC == 0) into the queue. When we send
+        // the first RTP or RTCP packet the real crypto context will be created.
+        // Refer to putData(), sendImmediate() in ccrtp's outqueue.cpp and
+        // takeinControlPacket() in ccrtp's control.cpp.
+        //
+         setOutQueueCryptoContext(senderCryptoContext);
+         setOutQueueCryptoContextCtrl(senderCryptoContextCtrl);
     }
     if (part == ForReceiver) {
         // To decrypt packets: intiator uses responder keys,
@@ -472,6 +488,18 @@ bool ZrtpQueue::srtpSecretsReady(SrtpSecret_t* secrets, EnableSecurity part)
                     authKeyLen,                              // authentication key len
                     secrets->respSaltLen / 8,                // session salt len
                     secrets->srtpAuthTagLen / 8);            // authentication tag len
+            recvCryptoContextCtrl = new CryptoContextCtrl(0,
+                  cipher,                                    // encryption algo
+                  authn,                                     // authtication algo
+                  (unsigned char*)secrets->keyResponder,     // Master Key
+                  secrets->respKeyLen / 8,                   // Master Key length
+                  (unsigned char*)secrets->saltResponder,    // Master Salt
+                  secrets->respSaltLen / 8,                  // Master Salt length
+                  secrets->respKeyLen / 8,                   // encryption keyl
+                  authKeyLen,                                // authentication key len
+                  secrets->respSaltLen / 8,                  // session salt len
+                  secrets->srtpAuthTagLen / 8);              // authentication tag len
+
         }
         else {
             recvCryptoContext = new CryptoContext(
@@ -488,30 +516,27 @@ bool ZrtpQueue::srtpSecretsReady(SrtpSecret_t* secrets, EnableSecurity part)
                     authKeyLen,                              // authentication key len
                     secrets->initSaltLen / 8,                // session salt len
                     secrets->srtpAuthTagLen / 8);            // authentication tag len
+            recvCryptoContextCtrl = new CryptoContextCtrl(0,
+                  cipher,                                    // encryption algo
+                  authn,                                     // authtication algo
+                  (unsigned char*)secrets->keyInitiator,     // Master Key
+                  secrets->initKeyLen / 8,                   // Master Key length
+                  (unsigned char*)secrets->saltInitiator,    // Master Salt
+                  secrets->initSaltLen / 8,                  // Master Salt length
+                  secrets->initKeyLen / 8,                   // encryption keyl
+                  authKeyLen,                                // authentication key len
+                  secrets->initSaltLen / 8,                  // session salt len
+                  secrets->srtpAuthTagLen / 8);              // authentication tag len
         }
         if (recvCryptoContext == NULL) {
             return false;
         }
-        // Create a SRTP crypto context for real SSRC input stream.
-        // If the sender didn't provide a SSRC just insert the template
-        // into the queue. After we received the first packet the real
-        // crypto context will be created.
+        // Insert the Crypto templates (SSRC == 0) into the queue. When we receive
+        // the first RTP or RTCP packet the real crypto context will be created.
+        // Refer to rtpDataPacket() above and takeinControlPacket in ccrtp's control.cpp.
         //
-        // Note: key derivation can be done at this time only if the
-        // key derivation rate is 0 (disabled). For ZRTP this is the
-        // case: the key derivation is defined as 2^48
-        // which is effectively 0.
-        if (peerSSRC != 0) {
-            pcc = recvCryptoContext->newCryptoContextForSSRC(peerSSRC, 0, 0L);
-            if (pcc == NULL) {
-                return false;
-            }
-            pcc->deriveSrtpKeys(0L);
-            setInQueueCryptoContext(pcc);
-        }
-        else {
-            setInQueueCryptoContext(recvCryptoContext);
-        }
+        setInQueueCryptoContext(recvCryptoContext);
+        setInQueueCryptoContextCtrl(recvCryptoContextCtrl);
     }
     return true;
 }
@@ -530,9 +555,11 @@ void ZrtpQueue::srtpSecretsOn(std::string c, std::string s, bool verified)
 void ZrtpQueue::srtpSecretsOff(EnableSecurity part) {
     if (part == ForSender) {
         removeOutQueueCryptoContext(NULL);
+        removeOutQueueCryptoContextCtrl(NULL);
     }
     if (part == ForReceiver) {
         removeInQueueCryptoContext(NULL);
+        removeInQueueCryptoContextCtrl(NULL);
     }
     if (zrtpUserCallback != NULL) {
         zrtpUserCallback->secureOff();
@@ -592,27 +619,27 @@ void ZrtpQueue::synchLeave() {
     synchLock.leave();
 }
 
-void ZrtpQueue::zrtpAskEnrollment(std::string info) {
+void ZrtpQueue::zrtpAskEnrollment(GnuZrtpCodes::InfoEnrollment  info) {
     if (zrtpUserCallback != NULL) {
         zrtpUserCallback->zrtpAskEnrollment(info);
     }
 }
 
-void ZrtpQueue::zrtpInformEnrollment(std::string info) {
+void ZrtpQueue::zrtpInformEnrollment(GnuZrtpCodes::InfoEnrollment  info) {
     if (zrtpUserCallback != NULL) {
         zrtpUserCallback->zrtpInformEnrollment(info);
     }
 }
 
-void ZrtpQueue::signSAS(std::string sas) {
+void ZrtpQueue::signSAS(uint8_t* sasHash) {
     if (zrtpUserCallback != NULL) {
-        zrtpUserCallback->signSAS(sas);
+        zrtpUserCallback->signSAS(sasHash);
     }
 }
 
-bool ZrtpQueue::checkSASSignature(std::string sas) {
+bool ZrtpQueue::checkSASSignature(uint8_t* sasHash) {
     if (zrtpUserCallback != NULL) {
-        return zrtpUserCallback->checkSASSignature(sas);
+        return zrtpUserCallback->checkSASSignature(sasHash);
     }
     return false;
 }
@@ -644,11 +671,6 @@ void ZrtpQueue::setAuxSecret(uint8* data, int32_t length)  {
         zrtpEngine->setAuxSecret(data, length);
 }
 
-void ZrtpQueue::setPbxSecret(uint8* data, int32 length)  {
-    if (zrtpEngine != NULL)
-        zrtpEngine->setPbxSecret(data, length);
-}
-
 void ZrtpQueue::setUserCallback(ZrtpUserCallback* ucb) {
     zrtpUserCallback = ucb;
 }
@@ -664,6 +686,13 @@ std::string ZrtpQueue::getHelloHash()  {
         return std::string();
 }
 
+std::string ZrtpQueue::getPeerHelloHash()  {
+    if (zrtpEngine != NULL)
+        return zrtpEngine->getPeerHelloHash();
+    else
+        return std::string();
+}
+
 std::string ZrtpQueue::getMultiStrParams()  {
     if (zrtpEngine != NULL)
         return zrtpEngine->getMultiStrParams();
@@ -693,15 +722,76 @@ void ZrtpQueue::acceptEnrollment(bool accepted) {
         zrtpEngine->acceptEnrollment(accepted);
 }
 
+std::string ZrtpQueue::getSasType() {
+    if (zrtpEngine != NULL)
+        return zrtpEngine->getSasType();
+    else
+        return NULL;
+}
+
+uint8_t* ZrtpQueue::getSasHash() {
+    if (zrtpEngine != NULL)
+        return zrtpEngine->getSasHash();
+    else
+        return NULL;
+}
+
+bool ZrtpQueue::sendSASRelayPacket(uint8_t* sh, std::string render) {
+
+    if (zrtpEngine != NULL)
+        return zrtpEngine->sendSASRelayPacket(sh, render);
+    else
+        return false;
+}
+
+bool ZrtpQueue::isMitmMode() {
+    return mitmMode;
+}
+
+void ZrtpQueue::setMitmMode(bool mitmMode) {
+    this->mitmMode = mitmMode;
+}
+
+bool ZrtpQueue::isEnrollmentMode() {
+    if (zrtpEngine != NULL)
+        return zrtpEngine->isEnrollmentMode();
+    else
+        return false;
+}
+
+void ZrtpQueue::setEnrollmentMode(bool enrollmentMode) {
+    if (zrtpEngine != NULL)
+        zrtpEngine->setEnrollmentMode(enrollmentMode);
+}
+
+void ZrtpQueue::setParanoidMode(bool yesNo) {
+        enableParanoidMode = yesNo;
+}
+
+bool ZrtpQueue::isParanoidMode() {
+        return enableParanoidMode;
+}
+
+bool ZrtpQueue::isPeerEnrolled() {
+    if (zrtpEngine != NULL)
+        return zrtpEngine->isPeerEnrolled();
+    else
+        return false;
+}
+
+void ZrtpQueue::setSignSas(bool sasSignMode) {
+    signSas = sasSignMode;
+}
+
 bool ZrtpQueue::setSignatureData(uint8* data, int32 length) {
     if (zrtpEngine != NULL)
         return zrtpEngine->setSignatureData(data, length);
     return 0;
 }
 
-int32 ZrtpQueue::getSignatureData(uint8* data) {
+const uint8* ZrtpQueue::getSignatureData() {
     if (zrtpEngine != NULL)
-        return zrtpEngine->getSignatureData(data);
+        return zrtpEngine->getSignatureData();
     return 0;
 }
 
@@ -711,18 +801,12 @@ int32 ZrtpQueue::getSignatureLength() {
     return 0;
 }
 
-void ZrtpQueue::setPBXEnrollment(bool yesNo) {
-    if (zrtpEngine != NULL)
-    zrtpEngine->setPBXEnrollment(yesNo);
-}
-
-
-int32 ZrtpQueue::getZid(uint8* data) {
+int32 ZrtpQueue::getPeerZid(uint8* data) {
     if (data == NULL)
         return 0;
 
     if (zrtpEngine != NULL)
-        return zrtpEngine->getZid(data);
+        return zrtpEngine->getPeerZid(data);
 
     return 0;
 }
