@@ -57,9 +57,12 @@ class __EXPORT ZrtpPacketConfirm : public ZrtpPacketBase {
         /// Normal destructor
         virtual ~ZrtpPacketConfirm();
 
-        /// Check is SAS verify flag is set
+        /// Check if SAS verify flag is set
         const bool isSASFlag()            { return confirmHeader->flags & 0x4; }
 
+        /// Check if PBXEnrollment flag is set
+        const bool isPBXEnrollment()      { return confirmHeader->flags & 0x8; }
+
         /// Get pointer to filler bytes (contains one bit of signature length)
         const uint8_t* getFiller()        { return confirmHeader->filler; }
 
@@ -75,12 +78,18 @@ class __EXPORT ZrtpPacketConfirm : public ZrtpPacketBase {
         /// Get pointer to initial hash chain (H0) data, fixed byte array
         uint8_t* getHashH0()              { return confirmHeader->hashH0; }
 
+        /// Get pointer to signature data, variable length, refer to getSignatureLength()
+        const uint8_t* getSignatureData() { return ((uint8_t*)&confirmHeader->expTime) + 4; }
+
         /// get the signature length in words
-        uint32_t getSignatureLength();
+        int32_t getSignatureLength();
 
         /// set SAS verified flag
         void setSASFlag()            { confirmHeader->flags |= 0x4; }
 
+        /// set setPBXEnrollment flag
+        void setPBXEnrollment()      { confirmHeader->flags |= 0x8; }
+
         /// Set MAC data, fixed length byte array
         void setHmac(uint8_t* text)  { memcpy(confirmHeader->hmac, text, sizeof(confirmHeader->hmac)); }
 
@@ -93,8 +102,11 @@ class __EXPORT ZrtpPacketConfirm : public ZrtpPacketBase {
         /// Set initial hash chain (H0) data, fixed length byte array
         void setHashH0(uint8_t* t)   { memcpy(confirmHeader->hashH0, t, sizeof(confirmHeader->hashH0)); }
 
+        /// Set signature data, length of the signature data in bytes and must be a multiple of 4.
+        bool setSignatureData(uint8_t* data, int32_t length);
+
         /// Set signature length in words
-        void setSignatureLength(uint32_t sl);
+        bool setSignatureLength(uint32_t sl);
 
     private:
         void initialize();
