@@ -65,6 +65,12 @@ enum EventDataType {
     ErrorPkt            ///< Error packet event
 };
 
+enum SecureSubStates {
+    Normal,
+    WaitSasRelayAck,
+    numberofSecureSubStates
+};
+
 /// A ZRTP state event
 typedef struct Event {
     EventDataType type; ///< Type of event
@@ -134,6 +140,25 @@ private:
      * variant of ZRTP. Refer to chapter 5.4.2 in the ZRTP specification.
      */
     bool multiStream;
+    
+    // Secure substate to handle SAS relay packets
+    SecureSubStates secSubstate;
+
+    /**
+     * Secure Sub state WaitSasRelayAck.
+     *
+     * This state belongs to the secure substates and handles
+     * SAS Relay Ack. 
+     *
+     * When entering this transition function
+     * - sentPacket contains Error packet, Error timer active
+     *
+     * Possible events in this state are:
+     * - timeout for sent SAS Relay packet: causes a resend check and repeat sending
+     *   of packet
+     * - SASRelayAck: Stop timer and switch to secure substate Normal.
+     */
+    bool subEvWaitRelayAck();
 
 public:
     /// Create a ZrtpStateClass
@@ -277,6 +302,19 @@ public:
      *    Value of the multi-stream mode flag.
      */
     bool isMultiStream();
+
+    /**
+     * Send a SAS relay packet.
+     *
+     * the functions stores sends the SAS relay packet and stores the pointer in
+     * the sentPacket variable to enable resending.
+     *
+     * The method switches to secure substate WaitSasRelayAck.
+     * 
+     * @param relay
+     *    Pointer to the SAS relay packet.
+     */
+    void sendSASRelay(ZrtpPacketSASrelay* relay);
 };
 
 /**
